
# This file was generated by PyScaffold from the git
# revision-control system data, or from the parent directory name of an
# unpacked source archive. Distribution tarballs contain a pre-generated copy
# of this file.

version_version = '0.0.6'
version_full = '106a3f682d8e1752ad8077dfadfb2a2295d814f0'
def get_versions(default=dict(), verbose=False):
    return dict(version=version_version, full=version_full)
